@@ -98,7 +98,7 @@ setMethod("glgm",
           formula, data,
           grid,
           covariates,
-          buffer=0)
+          buffer)
       
       callGeneric(
 			  formula = dataCov$formula, 
@@ -120,8 +120,9 @@ setMethod("glgm",
               formula, data, 
               grid, covariates, buffer)
           callGeneric(formula, 
-              dataCov$data@data, dataCov$grid, 
-              dataCov$covariates, ...)
+              data=dataCov$data@data, 
+							grid=dataCov$grid, 
+              covariates=dataCov$covariates, ...)
         }
     )
 
@@ -161,6 +162,9 @@ setMethod("glgm",
 	if(thedots$family=="gaussian") {
 		sdNames = unique(c(sdNames, "sdNugget"))
 	}
+	if(thedots$family=="gamma") {
+		sdNames = unique(c(sdNames, "gammaShape"))
+	}
 	
   # list of prior distributions
       if(any(names(priorCI)=='distributions')){
@@ -169,6 +173,8 @@ setMethod("glgm",
         priorDistributions = list()
       }
   
+	# priors for sd's (and precisions) 
+	
 	precPrior=list()
   
 	for(Dsd in sdNames) {
@@ -182,18 +188,41 @@ setMethod("glgm",
     
       if(all(c('shape','rate') %in% names(priorDistributions[[Dprec]]))) {
 
-        precPrior[[Dsd]] = c(
+        precPrior[[Dsd]] = list(
+						param = c(
             shape=as.numeric(priorDistributions[[Dprec]]['shape']), 
-            rate=as.numeric(priorDistributions[[Dprec]]['rate']))
+            rate=as.numeric(priorDistributions[[Dprec]]['rate'])),
+				prior = 'loggamma')
 
       } else {
-        precPrior[[Dsd]] = c(
+        precPrior[[Dsd]] = list(param=c(
             shape=priorDistributions[[Dprec]][1],
-            rate=priorDistributions[[Dprec]][2])
+            rate=priorDistributions[[Dprec]][2]),
+				prior = 'loggamma')
       }
     } else if(any(names(priorCI)==Dsd)) {
       # find distribution from interval supplied
 
+		
+		# if of length 1, it's pc prior u with alpha = 0.05
+		if(length(priorCI[[Dsd]])==1){
+			priorCI[[Dsd]] = c(
+					u=as.numeric(priorCI[[Dsd]]),
+					alpha = 0.05
+					)
+		}
+
+		if(!length(names(priorCI[[Dsd]])))
+			names(priorCI[[Dsd]]) = c('lower','upper')
+
+		if(all(c('u','alpha') %in% names(priorCI[[Dsd]]))) {
+			# pc priors
+			precPrior[[Dsd]] = list(
+					param=priorCI[[Dsd]],
+					prior = 'pc.prec')
+		} else {
+			# gamma prior
+		
     obj1 = sort(priorCI[[Dsd]]^-2)
 		cifun = function(pars) {
 				theci = 	pgamma(obj1, shape=pars[1], 
@@ -209,7 +238,11 @@ setMethod("glgm",
 		precPrior2=optim(c(.5,.5/mean(obj1)), cifun, 
 				lower=c(0.000001,0.0000001),method="L-BFGS-B")
 		names(precPrior2$par) = c("shape","rate")
-		precPrior[[Dsd]] = precPrior2$par 
+
+		precPrior[[Dsd]] = list(
+				param = precPrior2$par,
+				prior = 'loggamma')
+		
 				
  		#pgamma(obj1, shape= precPrior["shape"], rate=precPrior["rate"],log.p=F)
 		#pgamma(obj1, shape= precPrior["shape"], rate=precPrior["rate"],log.p=T)
@@ -220,8 +253,12 @@ setMethod("glgm",
  		#1/sqrt(qgamma(c(0.975,0.025), shape=precPrior["shape"], rate=precPrior["rate"]))
 		#priorCI$sd
 		
-		} else {
-			precPrior[[Dsd]] = c(shape=0.01, rate=0.01)
+		} # end gamma prior
+		} else { # no prior supplied
+			# default prior
+			precPrior[[Dsd]] = list(
+					param = c(shape=0.01, rate=0.01),
+					prior = 'loggamma')
 		}
 	}
 		
@@ -271,6 +308,21 @@ setMethod("glgm",
 		ratePrior = c(shape=0.01, rate=0.01)
 	}
 
+	# prior for gamma shape
+	# log-normal, priorCI is 4 standard deviations
+	if("gammaShape" %in% names(priorCI)) {
+		gammaShapePrior  = list(
+				prior='gaussian',
+				param=c(
+						mean=as.numeric(mean(log(priorCI$gammaShape))),
+						precision = as.numeric(abs(diff(log(priorCI$gammaShape)))[1]/4)^(-2)
+						)
+				)
+	} else {
+		gammaShapePrior = NULL
+	}
+
+
   spaceFormula = paste(".~.+ f(space, model='matern2d', ",
 				"nrow=", nrow(cells)+2*buffer, 
 				", ncol=", ncol(cells)+2*buffer,
@@ -280,9 +332,9 @@ setMethod("glgm",
 				      paste(ratePrior, collapse=","),
 				"), prior='loggamma'),",
 				"prec=list( param=c(",
-				paste(precPrior$sd, collapse=","),
-				"),prior='loggamma')",
-				" ) )" 
+				paste(precPrior$sd$param, collapse=","),
+				"), prior='",precPrior$sd$prior,"')",
+				" ) )", sep=""
 			)
 	
 	formula = update.formula(formula,	as.formula(spaceFormula))
@@ -393,10 +445,14 @@ formulaForLincombs = gsub("\\+[[:space:]]?$", "", formulaForLincombs)
 	# if model is gaussian, add prior for nugget
 	if(!is.null(precPrior$sdNugget)) {
 		forInla$control.family$hyper$prec =
-				list(prior="loggamma",
-						param=precPrior$sdNugget
+				list(prior=precPrior$sdNugget$prior,
+						param=precPrior$sdNugget$params
 				) 
 	}
+	if(!is.null(gammaShapePrior)) {
+		forInla$control.family$hyper$prec =
+				gammaShapePrior 
+	}
 	
 	
 	# get rid of some elements of forInla that aren't required
@@ -412,7 +468,6 @@ formulaForLincombs = gsub("\\+[[:space:]]?$", "", formulaForLincombs)
 			list(logfile="INLA is not installed. \n install splines, numDeriv, Rgraphviz, graph,\n fields, rgl, mvtnorm, multicore, pixmap,\n splancs, orthopolynom \n then see www.r-inla.org")
 	}
 
- 
 	
 	if(all(names(inlaResult)=="logfile"))
 		return(c(forInla, inlares=inlaResult))
@@ -446,27 +501,83 @@ formulaForLincombs = gsub("\\+[[:space:]]?$", "", formulaForLincombs)
 	)
 
   for(Dsd in names(precPrior)) {
+		if(precPrior[[Dsd]]$prior == 'loggamma'){
 		params[[Dsd]] = list(userPriorCI=priorCI[[Dsd]], 
 			priorCI = 1/sqrt(
 				qgamma(c(0.975,0.025), 
-						shape=precPrior[[Dsd]]["shape"], 
-						rate=precPrior[[Dsd]]["rate"])),
+						shape=precPrior[[Dsd]]$param["shape"], 
+						rate=precPrior[[Dsd]]$param["rate"])),
 					params.intern=precPrior[[Dsd]])
 	
 	precLim = 	qgamma(c(0.999,0.001), 
-			shape=precPrior[[Dsd]]["shape"], 
-			rate=precPrior[[Dsd]]["rate"])
+			shape=precPrior[[Dsd]]$param["shape"], 
+			rate=precPrior[[Dsd]]$param["rate"])
 	sdLim = 1/sqrt(precLim)
 	sdSeq = seq(min(sdLim), max(sdLim), len=1000)
 	precSeq = sdSeq^(-2)
 	params[[Dsd]]$prior=cbind(
 			x=sdSeq,
-			y=dgamma(precSeq, shape=precPrior[[Dsd]]["shape"], 
-					rate=precPrior[[Dsd]]["rate"]) *2* (precSeq)^(3/2) 
+			y=dgamma(precSeq, shape=precPrior[[Dsd]]$param["shape"], 
+					rate=precPrior[[Dsd]]$param["rate"]) *2* (precSeq)^(3/2) 
 	)
-
+	} else { # pc prior
+	
+		params[[Dsd]] = list(userPriorCI=priorCI[[Dsd]], 
+				priorCI = 1/sqrt(
+						INLA::inla.pc.qprec(c(0.975,0.025),  
+								u = precPrior[[Dsd]]$param['u'], 
+								alpha = precPrior[[Dsd]]$param['alpha'])
+				),
+		params.intern=precPrior[[Dsd]]$param)
+		
+		precLim = INLA::inla.pc.qprec(c(0.999,0.001),  
+						u = precPrior[[Dsd]]$param['u'], 
+						alpha = precPrior[[Dsd]]$param['alpha'])
+		sdLim = 1/sqrt(precLim)
+		sdSeq = seq(min(sdLim), max(sdLim), len=1000)
+		precSeq = sdSeq^(-2)
+		params[[Dsd]]$prior=cbind(
+				x=sdSeq,
+				y=INLA::inla.pc.dprec(precSeq, 
+						u = precPrior[[Dsd]]$param['u'], 
+						alpha = precPrior[[Dsd]]$param['alpha']
+					) * 2 * (precSeq)^(3/2) 
+		)
+		
+	}
 	}
 
+	if(!is.null(gammaShapePrior)) {
+		
+		paramsGammaShape = 	c(
+				gammaShapePrior$param["mean"], 
+				sd=as.numeric(1/sqrt(gammaShapePrior$param["precision"]))
+		)
+		
+		xLim = sort(exp(-qnorm(
+								c(0.999,0.001), 
+								mean=paramsGammaShape["mean"], 
+								sd=paramsGammaShape["sd"])
+						))
+		xSeq  = seq(xLim[1], xLim[2], len=1000)
+		
+		
+		params[['gammaShape']] = list(
+				userPriorCI = priorCI[['gammaShape']],
+				priorCI = sort(exp(-qnorm(c(0.975,0.025), 
+								mean=paramsGammaShape["mean"], 
+								sd=paramsGammaShape["sd"]))
+				),
+				params.intern=gammaShapePrior$param,
+				params = paramsGammaShape,
+				distribution = 'lognormal',
+				prior = cbind(
+						x=xSeq, 
+						y = stats::dlnorm(xSeq, meanlog = paramsGammaShape['mean'],sdlog = paramsGammaShape['sd'])
+						)
+				)
+	}
+	
 	
 	# random into raster
 # E exp(random)
@@ -511,7 +622,7 @@ inlaResult$summary.random[['space']][,"exp"] = temp
 	inlaResult$summary.lincomb.derived[,"exp"] = temp
 	
 	# E inv logit(lincombs)
-	if(length(grep("binomial",inlaResult$.args$family))) {
+	if(length(grep("logit",inlaResult$misc$linkfunctions$names))) {
 		temp=unlist(
 				lapply(inlaResult$marginals.lincomb.derived, function(qq) {
 							eqqx = exp(qq[,"x"])
@@ -604,7 +715,7 @@ params$summary = cbind(params$summary,
 				))
 )
 
-if(length(grep("binomial",inlaResult$.args$family))) {
+if(length(grep("logit",inlaResult$misc$linkfunctions$names))) {
 	params$summary = cbind(params$summary, 
 			meanInvLogit = unlist(
 					lapply(inlaResult$marginals.fixed, function(qq) {
@@ -623,6 +734,8 @@ thecols = paste(c("0.975", "0.5","0.025"), "quant", sep="")
 thesd = c(
 		sdNugget= grep("^Precision[[:print:]]*Gaussian observations$", 
 				names(inlaResult$marginals.hyperpar), value=TRUE),
+		gammaShape = grep("^Precision[[:print:]]*Gamma observations$", 
+				names(inlaResult$marginals.hyperpar), value=TRUE),
 		sd = grep("^Precision[[:print:]]*space$", 
 				names(inlaResult$marginals.hyperpar), value=TRUE)
 )
@@ -635,7 +748,7 @@ params$summary = rbind(params$summary,
 
 
 # convert precisions to standard deviations
-for(Dsd in names(thesd)) {
+for(Dsd in grep("gammaShape", names(thesd), invert=TRUE, value=TRUE)) {
 	
 	params[[Dsd]]$posterior=
 			inlaResult$marginals.hyperpar[[thesd[Dsd]]]
@@ -660,6 +773,32 @@ for(Dsd in names(thesd)) {
 	)
 }
 
+if(length(grep("gammaShape", names(thesd))) ) {
+	Dsd = 'gammaShape'
+params[[Dsd]]$posterior=
+		inlaResult$marginals.hyperpar[[thesd[Dsd]]]
+params[[Dsd]]$posterior[,"y"] = params[[Dsd]]$posterior[,"y"] *  
+		params[[Dsd]]$posterior[,"x"]^2 
+params[[Dsd]]$posterior[,"x"] = 1/params[[Dsd]]$posterior[,"x"]  
+params[[Dsd]]$posterior = params[[Dsd]]$posterior[
+		seq(dim(params[[Dsd]]$posterior)[1],1),]		
+
+params$summary[Dsd, thecols] = 
+		1/(inlaResult$summary.hyperpar[
+						thesd[Dsd],rev(thecols)])
+params$summary[Dsd,"mode"] = 
+    1/(inlaResult$summary.hyperpar[
+            thesd[Dsd],'mode'])
+
+
+params$summary[Dsd,"mean"] =sum(
+		1/(inlaResult$marginals.hyperpar[[thesd[Dsd]]][,"x"])*
+				c(0,diff(inlaResult$marginals.hyperpar[[thesd[Dsd]]][,"x"]))*
+				inlaResult$marginals.hyperpar[[thesd[Dsd]]][,"y"]
+)
+}
+
+
 # put range in summary, in units of distance, not numbers of cells
 thecolsFull =c("mean","sd",thecols,"mode") 
 params$summary["range",thecolsFull]=				
